@@ -12,7 +12,7 @@ from psycopg.errors import OperationalError, Error
 from jaaql.constants import KEY__position, KEY__file, KEY__application, KEY__error, KEY__error_row_number, KEY__error_query, \
     KEY__error_set, KEY__error_index, SQLStateJaaql, KEY__restrictions, REGEX__dmbs_object_name
 from jaaql.exceptions.jaaql_interpretable_handled_errors import DatabaseOperationalError, HandledProcedureError, UnhandledQueryError, UnhandledProcedureError, \
-    SingletonExpected
+    SingletonExpected, ExpectedParameterInQuery, MissingParameterInQuery
 from typing import Union
 
 
@@ -230,20 +230,24 @@ class InterpretJAAQL:
                     else:
                         cur_query = query_obj[KEY_query]
 
-                        last_query, found_parameter_dictionary = self.pre_prepare_statement(cur_query, parameters, require_presence=False)
+                        last_query, found_parameter_dictionary = self.pre_prepare_statement(cur_query, parameters, require_presence=False,
+                                                                                            query_name=query_key)
                         enc_parameter_dictionary = {}
                         encrypt_parameters = {key: val for key, val in parameters.items() if key not in found_parameter_dictionary}
                         if len(encrypt_parameters) != 0:
                             last_query, enc_parameter_dictionary = self.pre_prepare_statement(last_query, encrypt_parameters,
                                                                                               match_regex=REGEX_enc_query_argument,
                                                                                               encryption_key=encryption_key,
-                                                                                              require_presence=False)
+                                                                                              require_presence=False,
+                                                                                              query_name=query_key)
 
                         found_params = {**found_parameter_dictionary, **enc_parameter_dictionary}
                         check_unused -= set(found_params.keys())
 
                 if len(check_unused):
-                    raise HttpStatusException(ERR_unused_parameter % list(check_unused)[0], HTTPStatus.BAD_REQUEST)
+                    raise ExpectedParameterInQuery(ERR_unused_parameter % list(check_unused)[0], {
+                        "parameters": list(check_unused)
+                    })
 
             for query_key, query_obj in query.items():
                 exc_query_key = query_key
@@ -309,7 +313,7 @@ class InterpretJAAQL:
                     exc_row_number = cur_parameters.get(KEY__row_number)
                     last_query, found_parameter_dictionary = self.pre_prepare_statement(
                         cur_query, cur_parameters, for_prepare=do_prepare_only is not False and do_prepare_only is not None,
-                        skipped_singletons=skipped_singletons)
+                        skipped_singletons=skipped_singletons, query_name=query_key)
 
                     enc_parameter_dictionary = {}
 
@@ -319,7 +323,7 @@ class InterpretJAAQL:
                         last_query, enc_parameter_dictionary = self.pre_prepare_statement(
                             last_query, encrypt_parameters, match_regex=REGEX_enc_query_argument,
                             encryption_key=encryption_key, for_prepare=do_prepare_only is not False and do_prepare_only is not None,
-                            skipped_singletons=skipped_singletons)
+                            skipped_singletons=skipped_singletons, query_name=query_key)
 
                     last_query = self.encrypt_literals(last_query, encryption_key)
                     found_params = {**found_parameter_dictionary, **enc_parameter_dictionary}
@@ -664,7 +668,8 @@ class InterpretJAAQL:
         return new_query + query[last_end_match:]
 
     def pre_prepare_statement(self, query, parameters, match_regex: str = REGEX_query_argument, encryption_key: bytes = None,
-                              require_presence: bool = True, for_prepare: bool = False, skipped_singletons: list[str] = None):
+                              require_presence: bool = True, for_prepare: bool = False, skipped_singletons: list[str] = None,
+                              query_name: str = None):
         prepared = ""
         last_index = 0
         found_parameters = []
@@ -692,7 +697,7 @@ class InterpretJAAQL:
                         if skipped_singletons is not None and len(skipped_singletons) != 0:
                             missing_str += ". Is it possible the parameter exists in one of the following restricted singletons: " + ", ".join(
                                 missing_str) + "?"
-                        raise HttpStatusException(missing_str, HTTPStatus.BAD_REQUEST)
+                        raise MissingParameterInQuery(missing_str, query_name, {"parameter": match_str})
                     else:
                         continue
                 elif match_str not in parameters and match_str in MARKERS:
